from ctypes import c_void_p, c_char_p, c_size_t
from ctypes import c_uint8, c_uint, c_int, c_long, c_double, c_bool

import json

from ..lib import lib

from .content import ContentType, HTTP_CONTENT_TYPE_HTML
from .headers import http_header
from .status import http_status

# correctly deletes the response and all of its data
http_response_delete = lib.http_response_delete
http_response_delete.argtypes = [c_void_p]

# gets a new HTTP response ready to be used
http_response_get = lib.http_response_get
http_response_get.restype = c_void_p

# correctly disposes a HTTP response
http_response_return = lib.http_response_return
http_response_return.argtypes = [c_void_p]

# sets the http response's status code to be set in the header when compilling
http_response_set_status = lib.http_response_set_status
http_response_set_status.argtypes = [c_void_p, http_status]

# sets the response's header, it will replace the existing one
# the data will be deleted when the response gets deleted
http_response_set_header = lib.http_response_set_header
http_response_set_header.argtypes = [c_void_p, c_void_p, c_size_t]

# adds a new header to the response
# the headers will be handled when calling 
# http_response_compile () to generate a continuos header buffer
# returns 0 on success, 1 on error
http_response_add_header = lib.http_response_add_header
http_response_add_header.argtypes = [c_void_p, http_header, c_char_p]
http_response_add_header.restype = c_uint8

# adds a "Content-Type" header to the response
# returns 0 on success, 1 on error
http_response_add_content_type_header = lib.http_response_add_content_type_header
http_response_add_content_type_header.argtypes = [c_void_p, ContentType]
http_response_add_content_type_header.restype = c_uint8

# adds a "Content-Length" header to the response
# returns 0 on success, 1 on error
http_response_add_content_length_header = lib.http_response_add_content_length_header
http_response_add_content_length_header.argtypes = [c_void_p, c_size_t]
http_response_add_content_length_header.restype = c_uint8

# adds a "Content-Type" with value "application/json"
# adds a "Content-Length" header to the response
http_response_add_json_headers = lib.http_response_add_json_headers
http_response_add_json_headers.argtypes = [c_void_p, c_size_t]

# adds an "Access-Control-Allow-Origin" header to the response
# returns 0 on success, 1 on error
http_response_add_cors_header = lib.http_response_add_cors_header
http_response_add_cors_header.argtypes = [c_void_p, c_char_p]
http_response_add_cors_header.restype = c_uint8

# works like http_response_add_cors_header ()
# but takes a HttpOrigin instead of a c string
http_response_add_cors_header_from_origin = lib.http_response_add_cors_header_from_origin
http_response_add_cors_header_from_origin.argtypes = [c_void_p, c_void_p]
http_response_add_cors_header_from_origin.restype = c_uint8

# works like http_response_add_cors_header () but first
# checks if the domain matches any entry in the whitelist
# returns 0 on success, 1 on error
http_response_add_whitelist_cors_header = lib.http_response_add_whitelist_cors_header
http_response_add_whitelist_cors_header.argtypes = [c_void_p, c_char_p]
http_response_add_whitelist_cors_header.restype = c_uint8

# works like http_response_add_whitelist_cors_header ()
# but takes a HttpOrigin instead of a c string
http_response_add_whitelist_cors_header_from_origin = lib.http_response_add_whitelist_cors_header_from_origin
http_response_add_whitelist_cors_header_from_origin.argtypes = [c_void_p, c_void_p]
http_response_add_whitelist_cors_header_from_origin.restype = c_uint8

# checks if the HTTP request's "Origin" header value
# matches any domain in the whitelist
# then adds an "Access-Control-Allow-Origin" header to the response
# returns 0 on success, 1 on error
http_response_add_whitelist_cors_header_from_request = lib.http_response_add_whitelist_cors_header_from_request
http_response_add_whitelist_cors_header_from_request.argtypes = [c_void_p, c_void_p]
http_response_add_whitelist_cors_header_from_request.restype = c_uint8

# sets CORS related header "Access-Control-Allow-Credentials"
# this header is needed when a CORS request has an "Authorization" header
http_response_add_cors_allow_credentials_header = lib.http_response_add_cors_allow_credentials_header
http_response_add_cors_allow_credentials_header.argtypes = [c_void_p]
http_response_add_cors_allow_credentials_header.restype = c_uint8

# sets CORS related header "Access-Control-Allow-Methods"
# to be a list of available methods like "GET, HEAD, OPTIONS"
# this header is needed in preflight OPTIONS request's responses
# returns 0 on success, 1 on error
http_response_add_cors_allow_methods_header = lib.http_response_add_cors_allow_methods_header
http_response_add_cors_allow_methods_header.argtypes = [c_void_p, c_char_p]
http_response_add_cors_allow_methods_header.restype = c_uint8

# sets the response's data (body), it will replace the existing one
# the data will be deleted when the response gets deleted
http_response_set_data = lib.http_response_set_data
http_response_set_data.argtypes = [c_void_p, c_void_p, c_size_t]

# sets a reference to a data buffer to send
# data will not be copied into the response and will not be freed after use
# this method is similar to packet_set_data_ref ()
# returns 0 on success, 1 on error
http_response_set_data_ref = lib.http_response_set_data_ref
http_response_set_data_ref.argtypes = [c_void_p, c_void_p, c_size_t]
http_response_set_data_ref.restype = c_uint8

# creates a new http response with the specified status code
# ability to set the response's data (body); it will be copied to the response
# and the original data can be safely deleted 
http_response_create = lib.http_response_create
http_response_create.argtypes = [http_status, c_void_p, c_size_t]
http_response_create.restype = c_void_p

# merges the response header and the data into the final response
# returns 0 on success, 1 on error
http_response_compile = lib.http_response_compile
http_response_compile.argtypes = [c_void_p]
http_response_compile.restype = c_uint8

http_response_print = lib.http_response_print
http_response_print.argtypes = [c_void_p]

# send
# sends a response through the connection's socket
# returns 0 on success, 1 on error
http_response_send = lib.http_response_send
http_response_send.argtypes = [c_void_p, c_void_p]
http_response_send.restype = c_uint8

# expects a response with an already created header and data
# as this method will send both parts without the need of a continuos response buffer
# use this for maximun efficiency
# returns 0 on success, 1 on error
http_response_send_split = lib.http_response_send_split
http_response_send_split.argtypes = [c_void_p, c_void_p]
http_response_send_split.restype = c_uint8

# creates & sends a response through the connection's socket
# returns 0 on success, 1 on error
http_response_create_and_send = lib.http_response_create_and_send
http_response_create_and_send.argtypes = [http_status, c_void_p, c_size_t, c_void_p]
http_response_create_and_send.restype = c_uint8

def http_send_response (
	http_receive: c_void_p, status_code: http_status,
	body=None, body_len=None,
	content_type=HTTP_CONTENT_TYPE_HTML
):
	"""
	Creates and sends a HTTP response based on the supplied body
	# Parameters
	------------
	### http_receive: HttpReceive
		Reference to a HTTP receive instance
	### status_code: http_status
		The response's status code
	### body: dict, str, bytes
		Value(s) to send in the response's body
	### body_len: int
		Size of the body to send. If None it will be calculated
	### content_type: ContentType
		The response's body content type
	"""
	if (type (body) is dict):
		body_string = json.dumps (body)
		body_len = len (body_string)
		http_response_render_json (
			http_receive, status_code,
			body_string.encode ("utf-8"), body_len
		)

	else: 
		real_body = body
		if (type (body) is str):
			real_body = body.encode ("utf-8")
		if (body_len is None):
			body_len = len (body)

		response = http_response_create (
			status_code, real_body, body_len
		)

		http_response_add_content_type_header (response, content_type)
		http_response_add_content_length_header (response, body_len)

		http_response_compile (response)
		http_response_send (response, http_receive)
		http_response_delete (response)

# files
# opens the selected file and sends it back to the client
# takes care of generating the header based on the file values
# returns 0 on success, 1 on error
http_response_send_file = lib.http_response_send_file
http_response_send_file.argtypes = [c_void_p, http_status, c_char_p]
http_response_send_file.restype = c_uint8

# render
# sends the selected text back to the user
# this methods takes care of generating a repsonse with text/html content type
# returns 0 on success, 1 on error
http_response_render_text = lib.http_response_render_text
http_response_render_text.argtypes = [c_void_p, http_status, c_char_p, c_size_t]
http_response_render_text.restype = c_uint8

# sends the selected json back to the user
# this methods takes care of generating a repsonse with application/json content type
# returns 0 on success, 1 on error
http_response_render_json = lib.http_response_render_json
http_response_render_json.argtypes = [c_void_p, http_status, c_char_p, c_size_t]
http_response_render_json.restype = c_uint8

# videos
# handles the transmission of a video to the client
# returns 0 on success, 1 on error
http_response_handle_video = lib.http_response_handle_video
http_response_handle_video.argtypes = [c_void_p, c_char_p]
http_response_handle_video.restype = c_uint8

# json
# creates a HTTP response with the defined status code
# with a custom json message body
# returns a new HTTP response instance ready to be sent
http_response_create_json = lib.http_response_create_json
http_response_create_json.argtypes = [http_status, c_char_p, c_size_t]
http_response_create_json.restype = c_void_p

# creates a HTTP response with the defined status code and a data (body)
# with a json message of type { key: value } that is ready to be sent
# returns a new HTTP response instance
http_response_create_json_key_value = lib.http_response_create_json_key_value
http_response_create_json_key_value.argtypes = [http_status, c_char_p, c_char_p]
http_response_create_json_key_value.restype = c_void_p

# creates a HTTP response with the defined status code
# with a json body of type { "key": int_value }
# returns a new HTTP response instance ready to be sent
http_response_json_int_value = lib.http_response_json_int_value
http_response_json_int_value.argtypes = [http_status, c_char_p, c_int]
http_response_json_int_value.restype = c_void_p

# sends a HTTP response with custom status code
# with a json body of type { "key": int_value }
# returns 0 on success, 1 on error
http_response_json_int_value_send = lib.http_response_json_int_value_send
http_response_json_int_value_send.argtypes = [c_void_p, http_status, c_char_p, c_int]
http_response_json_int_value_send.restype = c_uint8

# creates a HTTP response with the defined status code
# with a json body of type { "key": large_int_value }
# returns a new HTTP response instance ready to be sent
http_response_json_large_int_value = lib.http_response_json_large_int_value
http_response_json_large_int_value.argtypes = [http_status, c_char_p, c_long]
http_response_json_large_int_value.restype = c_void_p

# sends a HTTP response with custom status code
# with a json body of type { "key": large_int_value }
# returns 0 on success, 1 on error
http_response_json_large_int_value_send = lib.http_response_json_large_int_value_send
http_response_json_large_int_value_send.argtypes = [c_void_p, http_status, c_char_p, c_long]
http_response_json_large_int_value_send.restype = c_uint8

# creates a HTTP response with the defined status code
# with a json body of type { "key": double_value }
# returns a new HTTP response instance ready to be sent
http_response_json_real_value = lib.http_response_json_real_value
http_response_json_real_value.argtypes = [http_status, c_char_p, c_double]
http_response_json_real_value.restype = c_void_p

# sends a HTTP response with custom status code
# with a json body of type { "key": double_value }
# returns 0 on success, 1 on error
http_response_json_real_value_send = lib.http_response_json_real_value_send
http_response_json_real_value_send.argtypes = [c_void_p, http_status, c_char_p, c_double]
http_response_json_real_value_send.restype = c_uint8

# creates a HTTP response with the defined status code
# with a json body of type { "key": bool_value }
# returns a new HTTP response instance ready to be sent
http_response_json_bool_value = lib.http_response_json_bool_value
http_response_json_bool_value.argtypes = [http_status, c_char_p, c_bool]
http_response_json_bool_value.restype = c_void_p

# sends a HTTP response with custom status code
# with a json body of type { "key": bool_value }
# returns 0 on success, 1 on error
http_response_json_bool_value_send = lib.http_response_json_bool_value_send
http_response_json_bool_value_send.argtypes = [c_void_p, http_status, c_char_p, c_bool]
http_response_json_bool_value_send.restype = c_uint8

# creates a HTTP response with the defined status code and a data (body)
# with a json message of type { msg: "your message" } ready to be sent
# returns a new HTTP response instance
http_response_json_msg = lib.http_response_json_msg
http_response_json_msg.argtypes = [http_status, c_char_p]
http_response_json_msg.restype = c_void_p

# creates and sends a HTTP json message response
# with the defined status code & message
# returns 0 on success, 1 on error
http_response_json_msg_send = lib.http_response_json_msg_send
http_response_json_msg_send.argtypes = [c_void_p, http_status, c_char_p]
http_response_json_msg_send.restype = c_uint8

# creates a HTTP response with the defined status code and a data (body)
# with a json message of type { error: "your error message" } ready to be sent
# returns a new HTTP response instance
http_response_json_error = lib.http_response_json_error
http_response_json_error.argtypes = [http_status, c_char_p]
http_response_json_error.restype = c_void_p

# creates and sends a HTTP json error response
# with the defined status code & message
# returns 0 on success, 1 on error
http_response_json_error_send = lib.http_response_json_error_send
http_response_json_error_send.argtypes = [c_void_p, http_status, c_char_p]
http_response_json_error_send.restype = c_uint8

# creates a HTTP response with the defined status code and a data (body)
# with a json meesage of type { key: value } ready to be sent
# returns a new HTTP response instance
http_response_json_key_value = lib.http_response_json_key_value
http_response_json_key_value.argtypes = [http_status, c_char_p, c_char_p]
http_response_json_key_value.restype = c_void_p

# creates and sends a HTTP custom json response
# with the defined status code & key-value
# returns 0 on success, 1 on error
http_response_json_key_value_send = lib.http_response_json_key_value_send
http_response_json_key_value_send.argtypes = [c_void_p, http_status, c_char_p, c_char_p]
http_response_json_key_value_send.restype = c_uint8

# creates a http response with the defined status code
# and the body with the custom json
http_response_json_custom = lib.http_response_json_custom
http_response_json_custom.argtypes = [http_status, c_char_p]
http_response_json_custom.restype = c_void_p

# creates and sends a http custom json response with the defined status code
# returns 0 on success, 1 on error
http_response_json_custom_send = lib.http_response_json_custom_send
http_response_json_custom_send.argtypes = [c_void_p, http_status, c_char_p]
http_response_json_custom_send.restype = c_uint8

# creates a http response with the defined status code
# and the body with a reference to a custom json
http_response_json_custom_reference = lib.http_response_json_custom_reference
http_response_json_custom_reference.argtypes = [http_status, c_char_p, c_size_t]
http_response_json_custom_reference.restype = c_void_p

# creates and sends a http custom json reference response
# with the defined status code
# returns 0 on success, 1 on error
http_response_json_custom_reference_send = lib.http_response_json_custom_reference_send
http_response_json_custom_reference_send.argtypes = [c_void_p, http_status, c_char_p, c_size_t]
http_response_json_custom_reference_send.restype = c_uint8



from ctypes import c_double

from .lib import lib

timer_sleep_for_seconds = lib.timer_sleep_for_seconds
timer_sleep_for_seconds.argtypes = [c_double]

timer_get_current_time = lib.timer_get_current_time
timer_get_current_time.restype = c_double



import json

from ctypes import CFUNCTYPE, POINTER, cast, c_void_p
from ctypes import c_int, c_uint, c_uint8, c_char_p, c_bool, c_size_t

from ..lib import lib

from ..types.string import String

from ..cerver import CERVER_HANDLER_TYPE_THREADS, cerver_set_alias, cerver_create_web
from ..cerver import cerver_set_receive_buffer_size, cerver_set_thpool_n_threads
from ..cerver import cerver_set_handler_type, cerver_set_reusable_address_flags

from .alg import jwt_alg_t, JWT_ALG_NONE
from .headers import http_header
from .request import RequestMethod, http_request_get_decoded_data
from .response import http_response_json_custom_reference_send
from .route import HttpHandler, HttpRouteAuthType, HTTP_ROUTE_AUTH_TYPE_NONE
from .route import HttpDecodeData, HttpDeleteDecoded, AuthenticationHandler
from .route import http_route_create, http_route_child_add, http_route_set_auth
from .route import http_route_set_decode_data_into_json
from .status import http_status, HTTP_STATUS_OK

# types
CatchAllHandler = CFUNCTYPE (None, c_void_p, c_void_p)
NotFoundHandler = CFUNCTYPE (None, c_void_p, c_void_p)
UploadsFilenameGenerator = CFUNCTYPE (None, c_void_p, c_void_p)
UploadsDirnameGenerator = CFUNCTYPE (None, c_void_p, c_void_p)
HttpDeleteCustom = CFUNCTYPE (None, c_void_p)

# main
http_cerver_get = lib.http_cerver_get
http_cerver_get.argtypes = [c_void_p]
http_cerver_get.restype = c_void_p

def http_cerver_configuration (
	name: str, port=8080, connection_queue=10,
	buffer_size=4096, n_threads=4,
	reusable_address_flags=True
) -> c_void_p:
	"""
	Creates a HTTP cerver with custom configuration
	# Parameters
	------------
	### name: str
		The name of the HTTP service, used for internal values.
	### port: int, optional
		Port where service will be exposed. Defaults to 8080.
	### connection_queue: int, optional
		Defaults to 10.
	### buffer_size: int, optional
		Defaults to 4096.
	### n_threads: int, optional
		Number of concurrent requests that service will be able to handle
		Defaults to 4.
	### reusable_address_flags: bool, optional
		Defaults to True.
	# Returns
	------------
	Reference to the created Cerver instance
	"""
	service_name = f"{name}-service"
	service = cerver_create_web (
		service_name.encode ("utf-8"), port, connection_queue
	)

	cerver_set_alias (service, name.encode ("utf-8"))

	cerver_set_receive_buffer_size (service, buffer_size)
	cerver_set_thpool_n_threads (service, n_threads)
	cerver_set_handler_type (service, CERVER_HANDLER_TYPE_THREADS)
	cerver_set_reusable_address_flags (service, reusable_address_flags)

	return service

# public
http_static_path_set_auth = lib.http_static_path_set_auth
http_static_path_set_auth.argtypes = [c_void_p, HttpRouteAuthType]

http_cerver_static_path_add = lib.http_cerver_static_path_add
http_cerver_static_path_add.argtypes = [c_void_p, c_char_p]
http_cerver_static_path_add.restype = c_void_p

http_receive_public_path_remove = lib.http_receive_public_path_remove
http_receive_public_path_remove.argtypes = [c_void_p, c_char_p]
http_receive_public_path_remove.restype = c_uint8

# routes
http_cerver_set_main_route = lib.http_cerver_set_main_route
http_cerver_set_main_route.argtypes = [c_void_p, c_void_p]

http_cerver_route_register = lib.http_cerver_route_register
http_cerver_route_register.argtypes = [c_void_p, c_void_p]

http_cerver_set_catch_all_route = lib.http_cerver_set_catch_all_route
http_cerver_set_catch_all_route.argtypes = [c_void_p, CatchAllHandler]

http_cerver_set_not_found_handler = lib.http_cerver_set_not_found_handler
http_cerver_set_not_found_handler.argtypes = [c_void_p]

http_cerver_set_not_found_route = lib.http_cerver_set_not_found_route
http_cerver_set_not_found_route.argtypes = [c_void_p, NotFoundHandler]

def http_create_route (
	http_cerver: c_void_p, request_method: RequestMethod,
	route_path: str, handler: HttpHandler,
	auth_method=HTTP_ROUTE_AUTH_TYPE_NONE
) -> c_void_p:
	"""
	Creates and registers a new HTTP route
	# Parameters
	------------
	### http_cerver: HttpCerver
		Reference to a HttpCerver instance.
	### request_method: RequestMethod
		The method this route will handle (GET, POST, ...)
	### route_path: str
		The actual route path.
	### handler: CFUNCTYPE (None, c_void_p, c_void_p)
		The callback handler method.
	### auth_method: HttpRouteAuthType, optional
		How the route will handle authentication.
	# Returns
	------------
	Reference to the registered HttpRoute instance
	"""
	route = http_route_create (
		request_method, route_path.encode ("utf-8"), handler
	)

	if (auth_method != HTTP_ROUTE_AUTH_TYPE_NONE):
		http_route_set_auth (route, auth_method)
		http_route_set_decode_data_into_json (route)

	# register main route to HTTP cerver
	http_cerver_route_register (http_cerver, route)

	return route

def http_create_child_route (
	parent: c_void_p, request_method: RequestMethod,
	route_path: str, handler: HttpHandler,
	auth_method=HTTP_ROUTE_AUTH_TYPE_NONE
) -> c_void_p:
	"""
	Creates and registers a child route to a parent
	# Parameters
	------------
	### parent: HttpRoute
		The parent HTTP route instance
	### request_method: RequestMethod.
		The method this route will handler (GET, POST, ...)
	### route_path: str
		The actual route path.
	### handler: CFUNCTYPE (None, c_void_p, c_void_p)
		The callback handler method.
	### auth_method: HttpRouteAuthType, optional
		How the route will handle authentication.
	# Returns
	------------
	Reference to the child HttpRoute instance
	"""
	child = http_route_create (
		request_method, route_path.encode ("utf-8"), handler
	)
	
	if (auth_method != HTTP_ROUTE_AUTH_TYPE_NONE):
		http_route_set_auth (child, auth_method)
		http_route_set_decode_data_into_json (child)

	# register child route to its parent
	http_route_child_add (parent, child)

	return child

# uploads
http_cerver_set_uploads_path = lib.http_cerver_set_uploads_path
http_cerver_set_uploads_path.argtypes = [c_void_p, c_char_p]

http_cerver_generate_uploads_path = lib.http_cerver_generate_uploads_path

http_cerver_set_uploads_file_mode = lib.http_cerver_set_uploads_file_mode
http_cerver_set_uploads_file_mode.argtypes = [c_void_p, c_uint]

http_cerver_set_uploads_filename_generator = lib.http_cerver_set_uploads_filename_generator
http_cerver_set_uploads_filename_generator.argtypes = [c_void_p, UploadsFilenameGenerator]

http_cerver_set_default_uploads_filename_generator = lib.http_cerver_set_default_uploads_filename_generator
http_cerver_set_default_uploads_filename_generator.argtypes = [c_void_p]

http_cerver_set_uploads_dir_mode = lib.http_cerver_set_uploads_dir_mode
http_cerver_set_uploads_dir_mode.argtypes = [c_void_p, c_uint]

http_cerver_set_uploads_dirname_generator = lib.http_cerver_set_uploads_dirname_generator
http_cerver_set_uploads_dirname_generator.argtypes = [c_void_p, UploadsDirnameGenerator]

http_cerver_set_default_uploads_dirname_generator = lib.http_cerver_set_default_uploads_dirname_generator
http_cerver_set_default_uploads_dirname_generator.argtypes = [c_void_p]

http_cerver_set_uploads_delete_when_done = lib.http_cerver_set_uploads_delete_when_done
http_cerver_set_uploads_delete_when_done.argtypes = [c_void_p, c_bool]

# auth
http_jwt_get_bearer = lib.http_jwt_get_bearer
http_jwt_get_bearer.argtypes = [c_void_p]
http_jwt_get_bearer.restype = c_char_p

http_jwt_get_bearer_len = lib.http_jwt_get_bearer_len
http_jwt_get_bearer_len.argtypes = [c_void_p]
http_jwt_get_bearer_len.restype = c_size_t

http_jwt_get_json = lib.http_jwt_get_json
http_jwt_get_json.argtypes = [c_void_p]
http_jwt_get_json.restype = c_char_p

http_jwt_get_json_len = lib.http_jwt_get_json_len
http_jwt_get_json_len.argtypes = [c_void_p]
http_jwt_get_json_len.restype = c_size_t

# sets the jwt algorithm used for encoding & decoding jwt tokens
# the default value is JWT_ALG_HS256
http_cerver_auth_set_jwt_algorithm = lib.http_cerver_auth_set_jwt_algorithm
http_cerver_auth_set_jwt_algorithm.argtypes = [c_void_p, jwt_alg_t]

# sets the filename from where the jwt private key will be loaded
http_cerver_auth_set_jwt_priv_key_filename = lib.http_cerver_auth_set_jwt_priv_key_filename
http_cerver_auth_set_jwt_priv_key_filename.argtypes = [c_void_p, c_char_p]

# sets the filename from where the jwt public key will be loaded
http_cerver_auth_set_jwt_pub_key_filename = lib.http_cerver_auth_set_jwt_pub_key_filename
http_cerver_auth_set_jwt_pub_key_filename.argtypes = [c_void_p, c_char_p]

http_cerver_auth_jwt_new = lib.http_cerver_auth_jwt_new
http_cerver_auth_jwt_new.restype = c_void_p

http_cerver_auth_jwt_delete = lib.http_cerver_auth_jwt_delete
http_cerver_auth_jwt_delete.argtypes = [c_void_p]

http_cerver_auth_jwt_add_value = lib.http_cerver_auth_jwt_add_value
http_cerver_auth_jwt_add_value.argtypes = [c_void_p, c_char_p, c_char_p]

http_cerver_auth_jwt_add_value_bool = lib.http_cerver_auth_jwt_add_value_bool
http_cerver_auth_jwt_add_value_bool.argtypes = [c_void_p, c_char_p, c_bool]

http_cerver_auth_jwt_add_value_int = lib.http_cerver_auth_jwt_add_value_int
http_cerver_auth_jwt_add_value_int.argtypes = [c_void_p, c_char_p, c_int]

http_cerver_auth_generate_bearer_jwt_json = lib.http_cerver_auth_generate_bearer_jwt_json
http_cerver_auth_generate_bearer_jwt_json.argtypes = [c_void_p, c_void_p]
http_cerver_auth_generate_bearer_jwt_json.restype = c_uint8

http_cerver_auth_generate_bearer_jwt_json_with_value = lib.http_cerver_auth_generate_bearer_jwt_json_with_value
http_cerver_auth_generate_bearer_jwt_json_with_value.argtypes = [c_void_p, c_void_p, c_char_p, c_char_p]
http_cerver_auth_generate_bearer_jwt_json_with_value.restype = c_uint8

def http_cerver_auth_configuration (
	http_cerver: c_void_p, jwt_algorithm=JWT_ALG_NONE,
	priv_key_filename=None, pub_key_filename=None
):
	"""
	Configurates the HTTP service's auth JWT algorithm and keys
	# Parameters
	------------
	### http_cerver: HttpCerver
		Reference to a HttpCerver instance.
	### jwt_algorithm: jwt_alg_t, optional
		The algorithm that will be used to manage JWTs.
		Must be the same as the one used to create the keys.
		Defaults to JWT_ALG_NONE.
	### priv_key_filename: string, optional.
		The private key filename.
	### pub_key_filename: string, optional.
		The public key filename.
	"""
	http_cerver_auth_set_jwt_algorithm (http_cerver, jwt_algorithm)
	if (jwt_algorithm != JWT_ALG_NONE):
		if (priv_key_filename):
			http_cerver_auth_set_jwt_priv_key_filename (
				http_cerver, priv_key_filename.encode ("utf-8")
			)

		if (pub_key_filename):
			http_cerver_auth_set_jwt_pub_key_filename (
				http_cerver, pub_key_filename.encode ("utf-8")
			)

def http_jwt_create (values={}):
	"""
	Creates a HttpJwt instance with custom values.
	Must be deleted with http_cerver_auth_jwt_delete () to avoid memory leaks
	# Parameters
	------------
	### values: dict, optional
		The actual Bearer JWT values. Defaults to {}.
	# Returns
	------------
	Reference to a HttpJwt instance
	"""
	http_jwt = http_cerver_auth_jwt_new ()
	for key in values:
		if (type (values[key]) == int):
			http_cerver_auth_jwt_add_value_int (
				http_jwt, key.encode ("utf-8"), values[key]
			)
		elif (type (values[key]) == bool):
			http_cerver_auth_jwt_add_value_bool (
				http_jwt, key.encode ("utf-8"), values[key]
			)
		elif (type (values[key]) == str):
			http_cerver_auth_jwt_add_value (
				http_jwt, key.encode ("utf-8"), values[key].encode ("utf-8")
			)

	return http_jwt


def http_jwt_create_and_send (
	http_receive: c_void_p,
	status_code=HTTP_STATUS_OK,
	values={}
):
	"""
	Creates and sends a Bearer JWT
	# Parameters
	------------
	### http_receive: HttpReceive
		Reference to a HttpReceive instance.
	### status_code: http_status, optional
		The HTTP response's status code. Defaults to HTTP_STATUS_OK.
	### values : dict, optional
		The actual Bearer JWT values. Defaults to {}.
	"""
	http_jwt = http_jwt_create (values)
	http_cerver_auth_generate_bearer_jwt_json (
		http_receive_get_cerver (http_receive), http_jwt
	)

	http_response_json_custom_reference_send (
		http_receive, status_code,
		http_jwt_get_json (http_jwt),
		http_jwt_get_json_len (http_jwt)
	)

	http_cerver_auth_jwt_delete (http_jwt)

def http_jwt_token_decode (request: c_void_p):
	"""
	Loads the decoded data from a JWT into a dict
	# Parameters
	------------
	### request: HttpRequest
		Reference to a HTTP request instance
	# Returns
	------------
	The JWT decoded data as a dict
	"""
	json_string = cast (http_request_get_decoded_data (request), c_char_p)
	return json.loads (json_string.value.decode ("utf-8"))

# origins
http_cerver_add_origin_to_whitelist = lib.http_cerver_add_origin_to_whitelist
http_cerver_add_origin_to_whitelist.argtypes = [c_void_p, c_char_p]
http_cerver_add_origin_to_whitelist.restype = c_uint8

http_cerver_print_origins_whitelist = lib.http_cerver_print_origins_whitelist
http_cerver_print_origins_whitelist.argtypes = [c_void_p]

# data
http_cerver_get_custom_data = lib.http_cerver_get_custom_data
http_cerver_get_custom_data.argtypes = [c_void_p]
http_cerver_get_custom_data.restype = c_void_p

http_cerver_set_custom_data = lib.http_cerver_set_custom_data
http_cerver_set_custom_data.argtypes = [c_void_p, c_void_p]

http_cerver_set_delete_custom_data = lib.http_cerver_set_delete_custom_data
http_cerver_set_delete_custom_data.argtypes = [c_void_p, HttpDeleteCustom]

http_cerver_set_default_delete_custom_data = lib.http_cerver_set_default_delete_custom_data
http_cerver_set_default_delete_custom_data.argtypes = [c_void_p]

# responses
http_cerver_add_responses_header = lib.http_cerver_add_responses_header
http_cerver_add_responses_header.argtypes = [c_void_p, http_header, c_char_p]
http_cerver_add_responses_header.restype = c_uint8

# stats
http_cerver_all_stats_print = lib.http_cerver_all_stats_print
http_cerver_all_stats_print.argtypes = [c_void_p]

# admin
http_cerver_enable_admin_routes = lib.http_cerver_enable_admin_routes
http_cerver_enable_admin_routes.argtypes = [c_void_p, c_bool]

http_cerver_enable_admin_info_route = lib.http_cerver_enable_admin_info_route
http_cerver_enable_admin_info_route.argtypes = [c_void_p, c_bool]

http_cerver_enable_admin_head_handlers = lib.http_cerver_enable_admin_head_handlers
http_cerver_enable_admin_head_handlers.argtypes = [c_void_p, c_bool]

http_cerver_enable_admin_options_handlers = lib.http_cerver_enable_admin_options_handlers
http_cerver_enable_admin_options_handlers.argtypes = [c_void_p, c_bool]

http_cerver_enable_admin_routes_authentication = lib.http_cerver_enable_admin_routes_authentication
http_cerver_enable_admin_routes_authentication.argtypes = [c_void_p, HttpRouteAuthType]

http_cerver_admin_routes_auth_set_decode_data = lib.http_cerver_admin_routes_auth_set_decode_data
http_cerver_admin_routes_auth_set_decode_data.argtypes = [c_void_p, HttpDecodeData, HttpDeleteDecoded]

http_cerver_admin_routes_auth_decode_to_json = lib.http_cerver_admin_routes_auth_decode_to_json
http_cerver_admin_routes_auth_decode_to_json.argtypes = [c_void_p]

http_cerver_admin_routes_set_authentication_handler = lib.http_cerver_admin_routes_set_authentication_handler
http_cerver_admin_routes_set_authentication_handler.argtypes = [c_void_p, AuthenticationHandler]

http_cerver_enable_admin_cors_headers = lib.http_cerver_enable_admin_cors_headers
http_cerver_enable_admin_cors_headers.argtypes = [c_void_p, c_bool]

http_cerver_admin_set_origin = lib.http_cerver_admin_set_origin
http_cerver_admin_set_origin.argtypes = [c_void_p, c_char_p]

http_cerver_register_admin_file_system = lib.http_cerver_register_admin_file_system
http_cerver_register_admin_file_system.argtypes = [c_void_p, c_char_p]

http_cerver_register_admin_worker = lib.http_cerver_register_admin_worker
http_cerver_register_admin_worker.argtypes = [c_void_p, c_void_p]

# handler
http_receive_get_cerver_receive = lib.http_receive_get_cerver_receive
http_receive_get_cerver_receive.argtypes = [c_void_p]
http_receive_get_cerver_receive.restype = c_void_p

http_receive_get_sock_fd = lib.http_receive_get_sock_fd
http_receive_get_sock_fd.argtypes = [c_void_p]
http_receive_get_sock_fd.restype = c_int

http_receive_get_cerver = lib.http_receive_get_cerver
http_receive_get_cerver.argtypes = [c_void_p]
http_receive_get_cerver.restype = c_void_p



from ctypes import POINTER, CFUNCTYPE, c_void_p, c_char_p
from ctypes import c_int, c_uint, c_uint8, c_size_t, c_bool

import distutils.util
import json

from ..lib import lib

from ..types.string import String

from .content import ContentType
from .headers import http_header
from .query import http_query_pairs_get_value

RequestMethod = c_int

REQUEST_METHOD_DELETE = 0
REQUEST_METHOD_GET = 1
REQUEST_METHOD_HEAD = 2
REQUEST_METHOD_POST = 3
REQUEST_METHOD_PUT = 4
REQUEST_METHOD_CONNECT = 5
REQUEST_METHOD_OPTIONS = 6
REQUEST_METHOD_TRACE = 7

REQUEST_METHOD_UNDEFINED = 8

RequestDeleteDecoded = CFUNCTYPE (None, c_void_p)
RequestDeleteCustom = CFUNCTYPE (None, c_void_p)

# getters
http_request_get_method = lib.http_request_get_method
http_request_get_method.argtypes = [c_void_p]
http_request_get_method.restype = RequestMethod

http_request_get_url = lib.http_request_get_url
http_request_get_url.argtypes = [c_void_p]
http_request_get_url.restype = POINTER (String)

http_request_get_query = lib.http_request_get_query
http_request_get_query.argtypes = [c_void_p]
http_request_get_query.restype = POINTER (String)

http_request_get_query_params = lib.http_request_get_query_params
http_request_get_query_params.argtypes = [c_void_p]
http_request_get_query_params.restype = c_void_p

http_request_get_n_params = lib.http_request_get_n_params
http_request_get_n_params.argtypes = [c_void_p]
http_request_get_n_params.restype = c_uint

http_request_get_param_at_idx = lib.http_request_get_param_at_idx
http_request_get_param_at_idx.argtypes = [c_void_p, c_uint]
http_request_get_param_at_idx.restype = POINTER (String)

http_request_get_header = lib.http_request_get_header
http_request_get_header.argtypes = [c_void_p, http_header]
http_request_get_header.restype = POINTER (String)

http_request_get_custom_headers_count = lib.http_request_get_custom_headers_count
http_request_get_custom_headers_count.argtypes = [c_void_p]
http_request_get_custom_headers_count.restype = c_size_t

http_request_get_custom_header = lib.http_request_get_custom_header
http_request_get_custom_header.argtypes = [c_void_p, c_char_p]
http_request_get_custom_header.restype = c_char_p

http_request_get_content_type = lib.http_request_get_content_type
http_request_get_content_type.argtypes = [c_void_p]
http_request_get_content_type.restype = ContentType

http_request_get_content_type_string = lib.http_request_get_content_type_string
http_request_get_content_type_string.argtypes = [c_void_p]
http_request_get_content_type_string.restype = POINTER (String)

http_request_content_type_is_json = lib.http_request_content_type_is_json
http_request_content_type_is_json.argtypes = [c_void_p]
http_request_content_type_is_json.restype = c_bool

http_request_get_decoded_data = lib.http_request_get_decoded_data
http_request_get_decoded_data.argtypes = [c_void_p]
http_request_get_decoded_data.restype = c_void_p

http_request_set_decoded_data = lib.http_request_set_decoded_data
http_request_set_decoded_data.argtypes = [c_void_p, c_void_p]

http_request_set_delete_decoded_data = lib.http_request_set_delete_decoded_data
http_request_set_delete_decoded_data.argtypes = [c_void_p, RequestDeleteDecoded]

http_request_set_default_delete_decoded_data = lib.http_request_set_default_delete_decoded_data
http_request_set_default_delete_decoded_data.argtypes = [c_void_p]

http_request_get_custom_data = lib.http_request_get_custom_data
http_request_get_custom_data.argtypes = [c_void_p]
http_request_get_custom_data.restype = c_void_p

http_request_set_custom_data = lib.http_request_set_custom_data
http_request_set_custom_data.argtypes = [c_void_p, c_void_p]

http_request_set_delete_custom_data = lib.http_request_set_delete_custom_data
http_request_set_delete_custom_data.argtypes = [c_void_p, RequestDeleteCustom]

http_request_set_default_delete_custom_data = lib.http_request_set_default_delete_custom_data
http_request_set_default_delete_custom_data.argtypes = [c_void_p]

http_request_get_body = lib.http_request_get_body
http_request_get_body.argtypes = [c_void_p]
http_request_get_body.restype = POINTER (String)

http_request_get_current_mpart = lib.http_request_get_current_mpart
http_request_get_current_mpart.argtypes = [c_void_p]
http_request_get_current_mpart.restype = c_void_p

http_request_get_n_files = lib.http_request_get_n_files
http_request_get_n_files.argtypes = [c_void_p]
http_request_get_n_files.restype = c_uint8

http_request_get_n_values = lib.http_request_get_n_values
http_request_get_n_values.argtypes = [c_void_p]
http_request_get_n_values.restype = c_uint8

http_request_get_dirname_len = lib.http_request_get_dirname_len
http_request_get_dirname_len.argtypes = [c_void_p]
http_request_get_dirname_len.restype = c_int

http_request_get_dirname = lib.http_request_get_dirname
http_request_get_dirname.argtypes = [c_void_p]
http_request_get_dirname.restype = c_char_p

http_request_set_dirname = lib.http_request_set_dirname

http_request_get_body_values = lib.http_request_get_body_values
http_request_get_body_values.argtypes = [c_void_p]
http_request_get_body_values.restype = c_void_p

def http_get_params_list (request: c_void_p) -> list:
	"""
	Gets all the HTTP request's params as a list
	# Parameters
	------------
	### request: HttpRequest
		Reference to a HTTP request instance
	# Returns
	------------
	The HTTP request's params as a list
	"""
	params = []
	try:
		n_params = http_request_get_n_params (request)
		for idx in range (n_params):
			params.append (
				http_request_get_param_at_idx (
					request, idx
				).contents.str.decode ("utf-8"))

	# error getting values from request
	except:
		params = None

	return params

def http_request_get_query_value (
	values: c_void_p, query_name: str
) -> str:
	"""
	Gets a query param from the HTTP request
	# Parameters
	------------
	### values: DoubleList <KeyValuePair>
		Key-value pairs parsed from x-www-form-urlencoded data or query params
	### query_name: string
		The key used to find a matching value
	# Returns
	------------
	The matching query value for the supplied key
	"""
	result = None

	value = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	if (value):
		result = value.contents.str.decode ("utf-8")

	return result

def http_request_get_int_query_value (
	values: c_void_p, query_name: str
) -> int:
	result = None

	value = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	if (value):
		result = int (value.contents.str.decode ("utf-8"))

	return result

def http_request_get_float_query_value (
	values: c_void_p, query_name: str
) -> float:
	result = None

	value = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	if (value):
		result = float (value.contents.str.decode ("utf-8"))

	return result

def http_request_get_bool_query_value (
	values: c_void_p, query_name: str
) -> bool:
	result = None

	value = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	if (value):
		result = bool (distutils.util.strtobool (value.contents.str.decode ("utf-8")))

	return result

def http_request_get_body_json (request: c_void_p) -> dict:
	"""
	Gets the HTTP request's JSON body as a dictionary
	# Parameters
	------------
	### request: HttpRequest
		Reference to a HTTP request instance
	# Returns
	------------
	A dict with the HTTP request's JSON body
	"""
	body_str = http_request_get_body (request)
	return json.loads (body_str.contents.str.decode ("utf-8"))

# headers
http_request_headers_print = lib.http_request_headers_print
http_request_headers_print.argtypes = [c_void_p]

http_request_headers_print_full = lib.http_request_headers_print_full
http_request_headers_print_full.argtypes = [c_void_p]

# query
http_request_query_params_print = lib.http_request_query_params_print
http_request_query_params_print.argtypes = [c_void_p]

http_request_query_params_get_value = lib.http_request_query_params_get_value
http_request_query_params_get_value.argtypes = [c_void_p, c_char_p]
http_request_query_params_get_value.restype = POINTER (String)

# multi-parts
http_request_multi_parts_get = lib.http_request_multi_parts_get
http_request_multi_parts_get.argtypes = [c_void_p, c_char_p]
http_request_multi_parts_get.restype = c_void_p

http_request_multi_parts_get_value = lib.http_request_multi_parts_get_value
http_request_multi_parts_get_value.argtypes = [c_void_p, c_char_p]
http_request_multi_parts_get_value.restype = c_char_p

http_request_multi_parts_get_filename = lib.http_request_multi_parts_get_filename
http_request_multi_parts_get_filename.argtypes = [c_void_p, c_char_p]
http_request_multi_parts_get_filename.restype = c_char_p

http_request_multi_parts_get_saved_filename = lib.http_request_multi_parts_get_saved_filename
http_request_multi_parts_get_saved_filename.argtypes = [c_void_p, c_char_p]
http_request_multi_parts_get_saved_filename.restype = c_char_p

http_request_multi_parts_iter_start = lib.http_request_multi_parts_iter_start
http_request_multi_parts_iter_start.argtypes = [c_void_p]
http_request_multi_parts_iter_start.restype = c_bool

http_request_multi_parts_iter_get_next = lib.http_request_multi_parts_iter_get_next
http_request_multi_parts_iter_get_next.argtypes = [c_void_p]
http_request_multi_parts_iter_get_next.restype = c_void_p

http_request_multi_part_keep_files = lib.http_request_multi_part_keep_files
http_request_multi_part_keep_files.argtypes = [c_void_p]

http_request_multi_part_discard_files = lib.http_request_multi_part_discard_files
http_request_multi_part_discard_files.argtypes = [c_void_p]

http_request_multi_parts_print = lib.http_request_multi_parts_print
http_request_multi_parts_print.argtypes = [c_void_p]

http_request_multi_parts_files_print = lib.http_request_multi_parts_files_print
http_request_multi_parts_files_print.argtypes = [c_void_p]

# body
http_request_body_get_value = lib.http_request_body_get_value
http_request_body_get_value.argtypes = [c_void_p, c_char_p]
http_request_body_get_value.restype = POINTER (String)



from ctypes import c_void_p
import html

from .content import HTTP_CONTENT_TYPE_HTML
from .headers import HTTP_HEADER_LOCATION
from .response import http_response_create
from .response import http_response_add_header
from .response import http_response_add_content_type_header
from .response import http_response_add_content_length_header
from .response import http_response_compile
from .response import http_response_print
from .response import http_response_send
from .response import http_response_delete
from .status import http_status, HTTP_STATUS_SEE_OTHER

def redirect (http_receive: c_void_p, status: http_status, location: str):
	display_location = html.escape (location)
	actual_location = display_location.encode ("utf-8")

	body = ('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">\n'
	"<title>Redirecting...</title>\n"
	"<h1>Redirecting...</h1>\n"
	"<p>You should be redirected automatically to target URL: "
	f'<a href="{display_location}">{display_location}</a>. If'
	" not click the link.")

	actual_body = body.encode ("utf-8")
	body_len = len (actual_body)

	response = http_response_create (status, actual_body, body_len)
	if (response):
		http_response_add_content_type_header (response, HTTP_CONTENT_TYPE_HTML)
		http_response_add_content_length_header (response, body_len)

		http_response_add_header (response, HTTP_HEADER_LOCATION, actual_location)

		http_response_compile (response)

		# http_response_print (response)

		http_response_send (response, http_receive)
		http_response_delete (response)



from ctypes import CFUNCTYPE, c_void_p, c_size_t

from ..lib import lib

ThreadWork = CFUNCTYPE (None, c_void_p)

thread_create_detached = lib.thread_create_detached
thread_create_detached.argtypes = [ThreadWork, c_void_p]
thread_create_detached.restype = c_size_t

thread_mutex_new = lib.thread_mutex_new
thread_mutex_new.restype = c_void_p

thread_mutex_delete = lib.thread_mutex_delete
thread_mutex_delete.argtypes = [c_void_p]

thread_mutex_lock = lib.thread_mutex_lock
thread_mutex_lock.argtypes = [c_void_p]

thread_mutex_unlock = lib.thread_mutex_unlock
thread_mutex_unlock.argtypes = [c_void_p]

thread_cond_new = lib.thread_cond_new
thread_cond_new.restype = c_void_p

thread_cond_delete = lib.thread_cond_delete
thread_cond_delete.argtypes = [c_void_p]



from .lib import lib

from .utils.log import LOG_TYPE_NONE, cerver_log_both

PYCERVER_VERSION = "0.9"
PYCERVER_VERSION_NAME = "Version 0.9"
PYCERVER_VERSION_DATE = "06/02/2022"
PYCERVER_VERSION_TIME = "13:53 CST"
PYCERVER_VERSION_AUTHOR = "Erick Salas"

version = {
	"id": PYCERVER_VERSION,
	"name": PYCERVER_VERSION_NAME,
	"date": PYCERVER_VERSION_DATE,
	"time": PYCERVER_VERSION_TIME,
	"author": PYCERVER_VERSION_AUTHOR
}

cerver_version_print_full = lib.cerver_version_print_full
cerver_version_print_version_id = lib.cerver_version_print_version_id
cerver_version_print_version_name = lib.cerver_version_print_version_name

def pycerver_version_print_full ():
	output = "\nPyCerver Version: {name}\n" \
		"Release Date: {date} - {time}\n" \
		"Author: {author}\n".format (**version)

	cerver_log_both (
		LOG_TYPE_NONE, LOG_TYPE_NONE,
		output.encode ("utf-8")
	)

def pycerver_version_print_version_id ():
	cerver_log_both (
		LOG_TYPE_NONE, LOG_TYPE_NONE,
		f"\nPyCerver Version ID: {version.id}\n".encode ("utf-8")
	)

def pycerver_version_print_version_name ():
	cerver_log_both (
		LOG_TYPE_NONE, LOG_TYPE_NONE,
		f"\nPyCerver Version: {version.name}\n".encode ("utf-8")
	)



from ctypes import c_void_p

import distutils.util

from ..files import file_exists
from ..files import IMAGE_TYPE_PNG, IMAGE_TYPE_JPEG
from ..files import files_image_get_type

from .multipart import http_multi_part_is_file
from .multipart import http_multi_part_get_filename
from .multipart import http_multi_part_get_generated_filename
from .multipart import http_multi_part_get_saved_filename
from .request import http_query_pairs_get_value
from .request import http_request_multi_parts_get
from .request import http_request_multi_parts_get_value
from .request import http_request_multi_parts_get_filename
from .request import http_request_multi_parts_get_saved_filename

def validate_query_exists (
	values: c_void_p, query_name: str, errors: dict
) -> str:
	result = None

	found = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	if (found):
		query_value = found.contents.str.decode ("utf-8")
		if (len (query_value) > 0):
			result = query_value

	if (result is None):
		errors[query_name] = f"Field {query_name} is required."

	return result

def validate_query_value (
	values: c_void_p, query_name: str, min_len: int, max_len: int, errors: dict
) -> str:
	result = None

	found = validate_query_exists (values, query_name, errors)
	if (found):
		value_len = len (found)
		if ((value_len >= min_len) and (value_len <= max_len)):
			result = found

		else:
			errors[query_name] = (
				"Field {0} must be between {1} and {2} characters long."
				.format	(query_name, min_len, max_len)
			)

	return result

def validate_query_int_value (
	values: c_void_p, query_name: str, errors: dict
) -> int:
	result = None

	found = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	if (found):
		try:
			result = int (found.contents.str.decode ("utf-8"))
		except ValueError:
			errors[query_name] = f"Field {query_name} is invalid."

	else:
		errors[query_name] = f"Field {query_name} is required."

	return result

def validate_query_int_value_with_default (
	values: c_void_p, query_name: str, default: int
) -> int:
	result = default

	found = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	if (found):
		try:
			result = int (found.contents.str.decode ("utf-8"))
		except ValueError:
			pass

	return result

def validate_query_float_value (
	values: c_void_p, query_name: str, errors: dict
) -> float:
	result = None

	found = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	if (found):
		try:
			result = float (found.contents.str.decode ("utf-8"))
		except ValueError:
			errors[query_name] = f"Field {query_name} is invalid."

	else:
		errors[query_name] = f"Field {query_name} is required."

	return result

def validate_query_float_value_with_default (
	values: c_void_p, query_name: str, default: float
) -> float:
	result = default

	found = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	if (found):
		try:
			result = float (found.contents.str.decode ("utf-8"))
		except ValueError:
			pass

	return result

def validate_query_bool_value (
	values: c_void_p, query_name: str, errors: dict
) -> bool:
	result = None

	found = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	if (found):
		try:
			result = bool (
				distutils.util.strtobool (found.contents.str.decode ("utf-8"))
			)
		except ValueError:
			errors[query_name] = f"Field {query_name} is invalid."

	else:
		errors[query_name] = f"Field {query_name} is required."

	return result

def validate_query_bool_value_with_default (
	values: c_void_p, query_name: str, default: bool
) -> bool:
	result = default

	found = http_query_pairs_get_value (values, query_name.encode ("utf-8"))
	if (found):
		try:
			result = bool (
				distutils.util.strtobool (found.contents.str.decode ("utf-8"))
			)
		except ValueError:
			pass

	return result

def validate_body_required_keys (values, body):
	if not all (k in body for k in values):
		raise Exception ("Missing key(s) in body")

def validate_body_value_exists (body: dict, value: str, errors: dict):
	result = None

	if (value in body):
		if (type (body[value]) is str):
			if (len (body[value]) > 0):
				result = body[value]

		else:
			result = body[value]

	if (result is None):
		errors[value] = f"Field {value} is required."

	return result

def validate_body_string_value_exists_ignore_size (
	body: dict, value: str, errors: dict
):
	result = None

	if (value in body):
		if (type (body[value]) is str):
			result = body[value]

	if (result is None):
		errors[value] = f"Field {value} is required."

	return result

def validate_body_value (
	body: dict, value: str, min_len: int, max_len: int, errors: dict
):
	result = None

	found = validate_body_value_exists (body, value, errors)
	if (found):
		value_len = len (found)
		if ((value_len >= min_len) and (value_len <= max_len)):
			result = found

		else:
			errors[value] = (
				"Field {0} must be between {1} and {2} characters long."
				.format	(value, min_len, max_len)
			)

	return result

def validate_body_int_value_exists (
	body: dict, value: str, errors: dict
) -> int:
	result = None

	if (value in body):
		if (type (body[value]) == int):
			result = body[value]

	if (result is None):
		errors[value] = f"Field {value} is required."

	return result

def validate_body_float_value_exists (
	body: dict, value: str, errors: dict
) -> float:
	result = None

	if (value in body):
		if (type (body[value]) == float):
			result = body[value]

	if (result is None):
		errors[value] = f"Field {value} is required."

	return result

def validate_body_value_with_default (body: dict, value: str, default: dict):
	result = default

	if (value in body):
		result = body[value]

	return result

def validate_mparts_exists (request: c_void_p, value: str, errors: dict):
	result = None

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		if (len (found) > 0):
			result = found

	if (result is None):
		errors[value] = f"Field {value} is required."

	return result

def validate_mparts_value (
	request: c_void_p, value: str, min_len: int, max_len: int, errors: dict
):
	result = None

	found = validate_mparts_exists (request, value, errors)
	if (found):
		value_len = len (found)
		if ((value_len >= min_len) and (value_len <= max_len)):
			result = found

		else:
			errors[value] = (
				"Field {0} must be between {1} and {2} characters long."
				.format	(value, min_len, max_len)
			)

	return result

def validate_mparts_value_with_default (
	request: c_void_p, value: str, default: str
):
	result = default

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		result = found.decode ("utf-8")

	return result

def validate_mparts_int (request: c_void_p, value: str, errors: dict) -> int:
	result = None

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = int (found)
		except ValueError:
			errors[value] = f"Field {value} is invalid."

	else:
		errors[value] = f"Field {value} is required."

	return result

def validate_mparts_int_with_default (
	request: c_void_p, value: str, default: int
) -> int:
	result = default

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = int (found)
		except ValueError:
			pass

	return result

def validate_mparts_float (
	request: c_void_p, value: str, errors: dict
) -> float:
	result = None

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = float (found)
		except ValueError:
			errors[value] = f"Field {value} is invalid."

	else:
		errors[value] = f"Field {value} is required."

	return result

def validate_mparts_float_with_default (
	request: c_void_p, value: str, default: float
) -> float:
	result = default

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = float (found)
		except ValueError:
			pass

	return result

def validate_mparts_bool (
	request: c_void_p, value: str, errors: dict
) -> bool:
	result = None

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = bool (distutils.util.strtobool (found))
		except ValueError:
			errors[value] = f"Field {value} is invalid."

	else:
		errors[value] = f"Field {value} is required."

	return result

def validate_mparts_bool_with_default (
	request: c_void_p, value: str, default: bool
) -> bool:
	result = default

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = bool (distutils.util.strtobool (found))
		except ValueError:
			pass

	return result

def validate_mparts_file_exists (
	request: c_void_p, value: str, errors: dict
) -> dict:
	values = None

	mpart = http_request_multi_parts_get (request, value.encode ("utf-8"))
	if (mpart):
		if (http_multi_part_is_file (mpart)):
			saved = http_multi_part_get_saved_filename (mpart)
			if (file_exists (saved)):
				values = {}
				original = http_multi_part_get_filename (mpart)
				values["original"] = original.decode ("utf-8")
				generated = http_multi_part_get_generated_filename (mpart)
				if (generated):
					values["generated"] = generated.decode ("utf-8")

				values["saved"] = saved.decode ("utf-8")

		else:
			errors[value] = f"Field {value} is not a file."

	else:
		errors[value] = f"File {value} is missing."

	return values

def validate_mparts_filename_exists (
	request: c_void_p, value: str, errors: dict
) -> str:
	result = None

	# get the value filename
	filename = http_request_multi_parts_get_filename (
		request, value.encode ("utf-8")
	)

	if (filename):
		result = filename.decode ("utf-8")

	else:
		errors[value] = f"File {value} is missing."

	return result

def validate_mparts_saved_filename_exists (
	request: c_void_p, value: str, errors: dict
) -> str:
	result = None

	# get the value filename
	filename = http_request_multi_parts_get_saved_filename (
		request, value.encode ("utf-8")
	)

	if (filename):
		result = filename.decode ("utf-8")

	else:
		errors[value] = f"File {value} is missing."

	return result

def validate_mparts_saved_file_exists (
	request: c_void_p, value: str, errors: dict
) -> str:
	result = None

	# get the value filename
	filename = http_request_multi_parts_get_saved_filename (
		request, value.encode ("utf-8")
	)

	if (filename):
		if (file_exists (filename)):
			result = filename.decode ("utf-8")

	if (result is None):
		errors[value] = f"File {value} is missing."

	return result

def validate_mparts_file_complete (
	request: c_void_p, value: str, errors: dict
) -> dict:
	values = None

	mpart = http_request_multi_parts_get (request, value.encode ("utf-8"))
	if (mpart):
		if (http_multi_part_is_file (mpart)):
			values = {}

			saved = http_multi_part_get_saved_filename (mpart)

			original = http_multi_part_get_filename (mpart)
			values["original"] = original.decode ("utf-8")
			generated = http_multi_part_get_generated_filename (mpart)
			if (generated):
				values["generated"] = generated.decode ("utf-8")

			values["saved"] = saved.decode ("utf-8")

		else:
			errors[value] = f"Field {value} is not a file."

	else:
		errors[value] = f"File {value} is missing."

	return values

def validate_mparts_file_is_image (
	request: c_void_p, image: str, errors: dict
) -> dict:
	values = None

	mpart = http_request_multi_parts_get (request, image.encode ("utf-8"))
	if (mpart):
		if (http_multi_part_is_file (mpart)):
			saved = http_multi_part_get_saved_filename (mpart)

			# validate file and get extension
			img_type = files_image_get_type (saved)
			if ((img_type == IMAGE_TYPE_PNG) or (img_type == IMAGE_TYPE_JPEG)):
				values = {}
				values["type"] = img_type
				original = http_multi_part_get_filename (mpart)
				values["original"] = original.decode ("utf-8")
				generated = http_multi_part_get_generated_filename (mpart)
				if (generated):
					values["generated"] = generated.decode ("utf-8")

				values["saved"] = saved.decode ("utf-8")

			else:
				errors[image] = f"File {image} is not png or jpeg."

		else:
			errors[image] = f"Field {image} is not a file."

	else:
		errors[image] = f"File {image} is missing."

	return values
